/**
 * 
 */
package es.postigo.dao.impl;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.sql.Types;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

import es.postigo.dao.UserDao;
import es.postigo.dto.User;
import es.postigo.dto.UserPk;
import es.postigo.exceptions.UserDaoException;
import es.postigo.jdbc.ResourceManager;
import es.postigo.utility.ConexionBD;

/**
 * @author informatica4
 *
 */
public class UserDaoImpl extends AbstractDataAccessObject implements UserDao
{
	/** 
	 * The factory class for this DAO has two versions of the create() method - one that
takes no arguments and one that takes a Connection argument. If the Connection version
is chosen then the connection will be stored in this attribute and will be used by all
calls to this DAO, otherwise a new Connection will be allocated for each operation.
	 */
	protected java.sql.Connection userConn;

	/** 
	 * All finder methods in this class use this SELECT constant to build their queries
	 */
	protected final String SQL_SELECT = "SELECT USER, PASS FROM " + getTableName() + "";

	/** 
	 * SQL INSERT statement for this table
	 */
	protected final String SQL_INSERT = "INSERT INTO " + getTableName() + " ( USER, PASS ) VALUES ( ?, ? )";

	/** 
	 * SQL UPDATE statement for this table
	 */
	protected final String SQL_UPDATE = "UPDATE " + getTableName() + " SET USER = ?, PASS = ? WHERE USER = ?";
	
	
	/** 
	 * SQL DELETE statement for this table
	 */
	protected final String SQL_DELETE = "DELETE FROM " + getTableName() + " WHERE USER = ? ";

	/** 
	 * Index of column COLUMN_USER
	 */
	protected static final int COLUMN_USER = 1;

	/** 
	 * Index of column COLUMN_PASS
	 */
	protected static final int COLUMN_PASS = 2;
		
	/** 
	 * Number of columns
	 */
	protected static final int NUMBER_OF_COLUMNS = 2;

	/** 
	 * Index of primary-key column COD_UBICACION and COD_PRODUCTO
	 */
	protected static final int PK_COLUMN_USER = 1;

	/** 
	 * Inserts a new row in the USER table.
	 */
	public UserPk insert(User dto) throws UserDaoException
	{
		long t1 = System.currentTimeMillis();
		// declare variables
		final boolean isConnSupplied = (userConn != null);
		Connection conn = null;
		PreparedStatement stmt = null;
		ResultSet rs = null;
		
		try {
			// get the user-specified connection or get a connection from the ResourceManager
			conn = isConnSupplied ? userConn : ResourceManager.getConnection();
		
			stmt = conn.prepareStatement( SQL_INSERT );
			if (dto.getLogin() != null) {
				stmt.setString( COLUMN_USER, dto.getLogin() );
			} else {
				stmt.setNull(COLUMN_USER, Types.INTEGER);
			}
			
			if (dto.getPass() != null) {
				stmt.setString( COLUMN_PASS, dto.getPass() );
			} else {
				stmt.setNull(COLUMN_PASS, Types.INTEGER);
			}
						
			System.out.println( "Executing " + SQL_INSERT + " with DTO: " + dto );
			int rows = stmt.executeUpdate();
			long t2 = System.currentTimeMillis();
			System.out.println( rows + " rows affected (" + (t2-t1) + " ms)" );
			return dto.createPk();
		}
		catch (SQLException _e) {
			_e.printStackTrace();
			throw new UserDaoException( "SQLException: " + _e.getMessage(), _e );
		}
		catch (Exception _e) {
			_e.printStackTrace();
			throw new UserDaoException( "Exception: " + _e.getMessage(), _e );
		}
		finally {
			ResourceManager.close(stmt);
			if (!isConnSupplied) {
				ResourceManager.close(conn);
			}
		
		}
		
	}

	/** 
	 * Updates a single row in the USER table.
	 */
	public void update(UserPk pk, User dto) throws UserDaoException
	{
		long t1 = System.currentTimeMillis();
		// declare variables
		final boolean isConnSupplied = (userConn != null);
		Connection conn = null;
		PreparedStatement stmt = null;
		
		try {
			// get the user-specified connection or get a connection from the ResourceManager
			conn = isConnSupplied ? userConn : ResourceManager.getConnection();
		
			System.out.println( "Executing " + SQL_UPDATE + " with DTO: " + dto );
			stmt = conn.prepareStatement( SQL_UPDATE );
			if (dto.getLogin() != null) {
				stmt.setString( COLUMN_USER, dto.getLogin() );
			} else {
				stmt.setNull(COLUMN_USER, Types.INTEGER);
			}
			if (dto.getPass() != null) {
				stmt.setString( COLUMN_PASS, dto.getPass() );
			} else {
				stmt.setNull(COLUMN_PASS, Types.INTEGER);
			}
			
			if (pk.getUser() != null) {
				stmt.setString( 3, pk.getUser() );
			} else {
				stmt.setNull(3, Types.INTEGER);
			}
		
			int rows = stmt.executeUpdate();
			long t2 = System.currentTimeMillis();
			System.out.println( rows + " rows affected (" + (t2-t1) + " ms)" );
		}
		catch (SQLException _e) {
			_e.printStackTrace();
			throw new UserDaoException( "SQLException: " + _e.getMessage(), _e );
		}
		catch (Exception _e) {
			_e.printStackTrace();
			throw new UserDaoException( "Exception: " + _e.getMessage(), _e );
		}
		finally {
			ResourceManager.close(stmt);
			if (!isConnSupplied) {
				ResourceManager.close(conn);
			}
		
		}
		
	}

	/** 
	 * Deletes a single row in the USER table.
	 */
	public void delete(String pStrUser) throws UserDaoException
	{
		long t1 = System.currentTimeMillis();
		// declare variables
		final boolean isConnSupplied = (userConn != null);
		Connection conn = null;
		PreparedStatement stmt = null;
		
		try {
			// get the user-specified connection or get a connection from the ResourceManager
			conn = isConnSupplied ? userConn : ResourceManager.getConnection();
		
			System.out.println( "Executing " + SQL_DELETE + " with PK: " + pStrUser );
			stmt = conn.prepareStatement( SQL_DELETE );
			if (pStrUser != null) {
				stmt.setString( 1, pStrUser );
			} else {
				stmt.setNull(1, Types.INTEGER);
			}
			
			int rows = stmt.executeUpdate();
			long t2 = System.currentTimeMillis();
			System.out.println( rows + " rows affected (" + (t2-t1) + " ms)" );
		}
		catch (SQLException _e) {
			_e.printStackTrace();
			throw new UserDaoException( "SQLException: " + _e.getMessage(), _e );
		}
		catch (Exception _e) {
			_e.printStackTrace();
			throw new UserDaoException( "Exception: " + _e.getMessage(), _e );
		}
		finally {
			ResourceManager.close(stmt);
			if (!isConnSupplied) {
				ResourceManager.close(conn);
			}
		
		}
		
	}

	/** 
	 * Returns the rows from the USER table that matches the specified primary-key value.
	 */
	public User findByPrimaryKey(UserPk pk) throws UserDaoException
	{
		return findByPrimaryKey( pk );
	}

	/** 
	 * Returns all rows from the USER table.
	 */
	public List<User> findAll() throws UserDaoException
	{
		return findByDynamicSelect( SQL_SELECT + " ORDER BY USER", null );
	}

	/** 
	 * Returns all rows from the USER table that match the criteria 'CODIGO = :codigo'.
	 */
	public User findByPrimaryKey(String pStrLogin) throws UserDaoException
	{
		List<User> ret = findByDynamicSelect( SQL_SELECT + " WHERE USER = ? ", new Object[] { pStrLogin } );
		return ret.size()==0 ? null : ret.get(0);
	}
	
	/**
	 * Method 'UserDaoImpl'
	 * 
	 */
	public UserDaoImpl()
	{
	}

	/**
	 * Method 'UserDaoImpl'
	 * 
	 * @param userConn
	 */
	public UserDaoImpl(final java.sql.Connection userConn)
	{
		this.userConn = userConn;
	}

	/**
	 * Method 'getTableName'
	 * 
	 * @return String
	 */
	public String getTableName()
	{
		return "USUARIO";
	}

	/** 
	 * Fetches a single row from the result set
	 */
	protected User fetchSingleResult(ResultSet rs) throws SQLException
	{
		if (rs.next()) {
			User dto = new User();
			populateDto( dto, rs);
			return dto;
		} else {
			return null;
		}
		
	}

	/** 
	 * Fetches multiple rows from the result set
	 */
	protected List<User> fetchMultiResults(ResultSet rs) throws SQLException
	{
		Collection resultList = new ArrayList();
		while (rs.next()) {
			User dto = new User();
			populateDto( dto, rs);
			resultList.add( dto );
		}
		return (List<User>) resultList;
	}

	/** 
	 * Populates a DTO with data from a ResultSet
	 */
	protected void populateDto(User dto, ResultSet rs) throws SQLException
	{
		dto.setLogin(rs.getString(COLUMN_USER));
		dto.setPass(rs.getString(COLUMN_PASS));
	}

	/** 
	 * Returns all rows from the USER table that match the specified arbitrary SQL statement
	 */
	public List<User> findByDynamicSelect(String sql, Object[] sqlParams) throws UserDaoException
	{
		// declare variables
		final boolean isConnSupplied = (userConn != null);
		Connection conn = null;
		PreparedStatement stmt = null;
		ResultSet rs = null;
		
		try {
			// get the user-specified connection or get a connection from the ResourceManager
			conn = isConnSupplied ? userConn : ResourceManager.getConnection();
		
			// construct the SQL statement
			final String SQL = sql;
		
		
			System.out.println( "Executing " + SQL );
			// prepare statement
			stmt = conn.prepareStatement( SQL );
		
			// bind parameters
			for (int i=0; sqlParams!=null && i<sqlParams.length; i++ ) {
				stmt.setObject( i+1, sqlParams[i] );
			}
		
		
			rs = stmt.executeQuery();
		
			// fetch the results
			return fetchMultiResults(rs);
		}
		catch (SQLException _e) {
			_e.printStackTrace();
			throw new UserDaoException( "SQLException: " + _e.getMessage(), _e );
		}
		catch (Exception _e) {
			_e.printStackTrace();
			throw new UserDaoException( "Exception: " + _e.getMessage(), _e );
		}
		finally {
			ResourceManager.close(rs);
			ResourceManager.close(stmt);
			if (!isConnSupplied) {
				ResourceManager.close(conn);
			}
		
		}
		
	}

	/** 
	 * Returns all rows from the USER table that match the specified arbitrary SQL statement
	 */
	public List<User> findByDynamicWhere(String sql, Object[] sqlParams) throws UserDaoException
	{
		// declare variables
		final boolean isConnSupplied = (userConn != null);
		Connection conn = null;
		PreparedStatement stmt = null;
		ResultSet rs = null;
		
		try {
			// get the user-specified connection or get a connection from the ResourceManager
			conn = isConnSupplied ? userConn : ResourceManager.getConnection();
		
			// construct the SQL statement
			final String SQL = SQL_SELECT + " WHERE " + sql;
		
		
			System.out.println( "Executing " + SQL );
			// prepare statement
			stmt = conn.prepareStatement( SQL );
		
			// bind parameters
			for (int i=0; sqlParams!=null && i<sqlParams.length; i++ ) {
				stmt.setObject( i+1, sqlParams[i] );
			}
		
		
			rs = stmt.executeQuery();
		
			// fetch the results
			return fetchMultiResults(rs);
		}
		catch (SQLException _e) {
			_e.printStackTrace();
			throw new UserDaoException( "SQLException: " + _e.getMessage(), _e );
		}
		catch (Exception _e) {
			_e.printStackTrace();
			throw new UserDaoException( "Exception: " + _e.getMessage(), _e );
		}
		finally {
			ResourceManager.close(rs);
			ResourceManager.close(stmt);
			if (!isConnSupplied) {
				ResourceManager.close(conn);
			}
		
		}
		
	}
	
	public List<User> findByDynamicWhere(String pstrSQL)
			throws UserDaoException {
		return findByDynamicWhere(pstrSQL, null);
	}

	/*public User findByPrimaryKey(String pStrUser) throws UserDaoException {
		User inv = null;
		Connection con;
		try {
			con = ConexionBD.obtieneConexion();
			Statement s = con.createStatement();
			String strSQL = "SELECT * FROM USUARIO WHERE USER = '" + pStrUser + "'";
			ResultSet rs = s.executeQuery(strSQL);
			if(rs.next()){
				System.out.println("ok");
				inv = new User();
			}
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}		
		return inv;
		//return findByPrimaryKey( codUbicacion, codProducto );
	}*/
	
	public List<User> findByDynamicSelect(String pstrSQL)
			throws UserDaoException {
		return findByDynamicSelect(pstrSQL, null);
	}	
	
}